package net.scott.minecraftvillagerdimensionmod.entity.custom;

import net.minecraft.class_1282;
import net.minecraft.class_1299;
import net.minecraft.class_1309;
import net.minecraft.class_1347;
import net.minecraft.class_1366;
import net.minecraft.class_1376;
import net.minecraft.class_1379;
import net.minecraft.class_1400;
import net.minecraft.class_1604;
import net.minecraft.class_1646;
import net.minecraft.class_1657;
import net.minecraft.class_1937;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_3213;
import net.minecraft.class_3222;
import net.minecraft.class_5132;
import net.minecraft.class_5134;
import net.minecraft.class_52;
import net.minecraft.class_5321;
import net.minecraft.class_7924;
import net.minecraft.entity.ai.goal.*;
import net.scott.minecraftvillagerdimensionmod.MinecraftVillagerDimensionMod;
import software.bernie.geckolib.animatable.GeoAnimatable;
import software.bernie.geckolib.animatable.GeoEntity;
import software.bernie.geckolib.animatable.instance.AnimatableInstanceCache;
import software.bernie.geckolib.animatable.instance.SingletonAnimatableInstanceCache;
import software.bernie.geckolib.animation.*;

public class EvilVillagerBossEntity extends class_1604 implements GeoEntity {

    // For Villager and Evil State
    private boolean isEvil = false;
    private AnimationController<GeoAnimatable> transformationController;

    // For loot table
    public static final class_5321<class_52> LOOT_TABLE_KEY = class_5321.method_29179(
            class_7924.field_50079,
            class_2960.method_60655(MinecraftVillagerDimensionMod.MOD_ID, "entities/evilvillager")
    );

    // For animations
    private AnimatableInstanceCache cache = new SingletonAnimatableInstanceCache(this);
    // For Boss Bar
    private class_3213 bossBar = null;

    // Super Function
    public EvilVillagerBossEntity(class_1299<? extends class_1604> entityType, class_1937 world) {
        super(entityType, world);
        this.bossBar = new class_3213(
                class_2561.method_43470("Aragog The Hidden"), // Boss bar title
                class_3213.Color.field_5785,           // Boss bar color
                class_3213.Style.field_5795       // Boss bar style
        );
        if(isEvil){
            this.bossBar.method_14091(true);
        }
        else{
            this.bossBar.method_14091(false); // Hide the boss bar initially
        }
    }

    // Standard Attributes Class
    public static class_5132.class_5133 createEvilVillagerBossAttributes() {
        return class_1604.method_26828()
                .method_26868(class_5134.field_23716, 200)
                .method_26868(class_5134.field_23719, 0.5f)
                .method_26868(class_5134.field_23724, 3.0f)
                .method_26868(class_5134.field_23721, 13);
    }

    // Goals Class
    @Override
    protected void method_5959() {
        // Transform when targeting a player
        this.field_6185.method_6277(0, new class_1400<>(this, class_1657.class, true));

        // Villager state actions
        if (isEvil) {
            this.field_6201.method_6277(1, new class_1366(this, 0.5F, true));
            this.field_6185.method_6277(2, new class_1400<>(this, class_1646.class, true));
        }
        this.field_6201.method_6277(3, new class_1379(this, 0.5F, 15));
        this.field_6201.method_6277(4, new class_1347(this));
        this.field_6201.method_6277(5, new class_1376(this));

    }

    // Method to trigger evil transformation
    private boolean triggerTransformation(class_1309 target) {
        if (!isEvil && target instanceof class_1657) {
            isEvil = true;
            System.out.println("Transformation triggered: Now evil!");

            if (transformationController != null) {
                transformationController.forceAnimationReset();
                transformationController.setAnimation(RawAnimation.begin().then("animation.model.evil_transform", Animation.LoopType.PLAY_ONCE));
            }

            // Update AI goals
            this.field_6201.method_35115().clear();
            this.method_5959();

            // Show boss bar after transformation
            if (!this.method_37908().field_9236) {
                System.out.println("Making boss bar visible");
                this.bossBar.method_14091(true);
                for (class_1657 player : this.method_37908().method_18456()) {
                    if (player instanceof class_3222) {
                        this.bossBar.method_14088((class_3222) player);
                    }
                }
            }

            return true;
        }
        return false;
    }

    @Override
    public void registerControllers(AnimatableManager.ControllerRegistrar controllers) {
        controllers.add(new AnimationController<GeoAnimatable>(this, "controller", 0, this::predicate));
        controllers.add(new AnimationController<GeoAnimatable>(this, "attackcontroller", 0, this::attackpredicate));
        transformationController = new AnimationController<>(this, "transformcontroller", 0, this::transformationPredicate);
        controllers.add(transformationController);
    }

    private PlayState transformationPredicate(AnimationState<GeoAnimatable> tAnimationState) {
        if (isEvil) {
            return PlayState.STOP;
        }

        tAnimationState.getController().setAnimation(RawAnimation.begin().then("animation.model.evil_transform", Animation.LoopType.PLAY_ONCE));
        return PlayState.CONTINUE;
    }

    // For attack animation
    private PlayState attackpredicate(AnimationState<GeoAnimatable> tAnimationState) {
        if (this.field_6252 && tAnimationState.getController().getAnimationState().equals(AnimationController.State.STOPPED)) {
            this.field_6252 = false;

            if (isEvil) {
                tAnimationState.getController().forceAnimationReset();
                tAnimationState.getController().setAnimation(RawAnimation.begin().then("animation.model.attack_evil", Animation.LoopType.PLAY_ONCE));
            }
        }
        return PlayState.CONTINUE;
    }

    // Prevents the boss from despawning
    @Override
    public boolean method_5974(double distanceSquared) {
        return false; // Prevents natural despawning
    }


    // For Walk and Idle
    private PlayState predicate(AnimationState<GeoAnimatable> tAnimationState) {
        if (isEvil) {
            if (tAnimationState.isMoving()) {
                tAnimationState.getController().setAnimation(RawAnimation.begin().then("animation.model.walk_evil", Animation.LoopType.LOOP));
            } else {
                tAnimationState.getController().setAnimation(RawAnimation.begin().then("animation.model.idle_evil", Animation.LoopType.LOOP));
            }
        } else {
            if (tAnimationState.isMoving()) {
                tAnimationState.getController().setAnimation(RawAnimation.begin().then("animation.model.villager_walk", Animation.LoopType.LOOP));
            } else {
                tAnimationState.getController().setAnimation(RawAnimation.begin().then("animation.model.villager_idle", Animation.LoopType.LOOP));
            }
        }
        return PlayState.CONTINUE;
    }

    @Override
    public AnimatableInstanceCache getAnimatableInstanceCache() {
        return cache;
    }

    // Mostly for the boss bar
    @Override
    public void method_5773() {
        super.method_5773();

        if (!isEvil) {
            for (class_1657 player : this.method_37908().method_18456()) {
                if (this.method_5858(player) < 9) { // 3-block radius
                    triggerTransformation(player);
                    break;
                }
            }
        }

        // Only update the boss bar if the entity is evil
        if (isEvil && !this.method_37908().field_9236 && this.method_5805()) {
            this.bossBar.method_14091(true);
            this.bossBar.method_5408(this.method_6032() / this.method_6063());
        }
    }
    // For the boss bar
    @Override
    public void method_5837(class_3222 player) {
        super.method_5837(player);
        // Add player to boss bar
        if (isEvil) {
            this.bossBar.method_14088(player);
        }
    }
    // For the boss bar
    @Override
    public void method_5742(class_3222 player) {
        super.method_5742(player);
        this.bossBar.method_14089(player); // Remove player from boss bar
    }
    @Override
    public void method_6078(class_1282 source) {
        super.method_6078(source);
        if (!this.method_37908().field_9236) {
            this.bossBar.method_14094();
        }
    }
    // For loot table
    @Override
    protected class_5321<class_52> method_5991() {
        return LOOT_TABLE_KEY;
    }
}
