package net.scott.minecraftvillagerdimensionmod.entity.custom;

import net.minecraft.class_1299;
import net.minecraft.class_1307;
import net.minecraft.class_1308;
import net.minecraft.class_1335;
import net.minecraft.class_1347;
import net.minecraft.class_1352;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_2960;
import net.minecraft.class_3730;
import net.minecraft.class_5132;
import net.minecraft.class_5134;
import net.minecraft.class_5425;
import net.minecraft.entity.ai.goal.*;
import org.jetbrains.annotations.Nullable;
import software.bernie.geckolib.animatable.GeoAnimatable;
import software.bernie.geckolib.animatable.GeoEntity;
import software.bernie.geckolib.animatable.instance.AnimatableInstanceCache;
import software.bernie.geckolib.animatable.instance.SingletonAnimatableInstanceCache;
import software.bernie.geckolib.animation.*;

import java.util.EnumSet;
import java.util.Random;

public class MonarchButterfly extends class_1307 implements GeoEntity {
    private AnimatableInstanceCache cache = new SingletonAnimatableInstanceCache(this);
    private static final Random RANDOM = new Random();


    // DataTracker to sync the texture
    private final class_2960 butterflyTexture;
    public MonarchButterfly(class_1299<? extends class_1307> entityType, class_1937 world) {
        super(entityType, world);
        this.field_6207 = new ButterflyMoveControl(this);
        this.butterflyTexture = selectRandomTexture();
    }

    @Override
    protected void method_5959() {
        this.field_6201.method_6277(0, new RandomFlyGoal(this, 0.2F));
        //this.goalSelector.add(5, new WanderAroundGoal(this, 0.5F, 3));
        this.field_6201.method_6277(1, new class_1347(this));
    }

    public static class_5132.class_5133 createMonarchButterflyAttributes() {
        return class_1308.method_26828()
                .method_26868(class_5134.field_23716, 5.0)
                .method_26868(class_5134.field_23720, 0.6F)
                .method_26868(class_5134.field_23719, 0.3F)
                .method_26868(class_5134.field_23717, 48.0);
    }

    // Random texture selection logic with rarity weights
    private class_2960 selectRandomTexture() {
        class_2960[] textures = new class_2960[]{
                class_2960.method_60655("minecraftvillagerdimensionmod", "textures/entity/monarch_butterfly_texture.png"),      // Common
                class_2960.method_60655("minecraftvillagerdimensionmod", "textures/entity/monarch_butterfly_texture_2.png"),    // Rare
                class_2960.method_60655("minecraftvillagerdimensionmod", "textures/entity/monarch_butterfly_texture_3.png"),    // Epic
                class_2960.method_60655("minecraftvillagerdimensionmod", "textures/entity/monarch_butterfly_texture_4.png")     // Legendary
        };

        int[] weights = {50, 30, 15, 4}; // Rarity distribution
        int totalWeight = 0;

        for (int weight : weights) {
            totalWeight += weight;
        }

        int randomValue = RANDOM.nextInt(totalWeight);
        for (int i = 0; i < weights.length; i++) {
            if (randomValue < weights[i]) {
                return textures[i];
            }
            randomValue -= weights[i];
        }

        return textures[0]; // Fallback to common
    }

    @Nullable
    public class_2960 getButterflyTexture() {
        return this.butterflyTexture;
    }

    public static boolean isOnGroundPredicate(class_1299<MonarchButterfly> type, class_5425 world, class_3730 spawnReason, class_2338 pos, net.minecraft.class_5819 random) {
        // Check if the block below the spawn position is a solid block.
        return world.method_8320(pos.method_10074()).method_26212(world, pos.method_10074());
    }

    @Override
    public void registerControllers(AnimatableManager.ControllerRegistrar controllers) {
        controllers.add(new AnimationController<GeoAnimatable>(this, "controller", 0, this::predicate));
    }

    private PlayState predicate(AnimationState<GeoAnimatable> geoAnimatableAnimationState) {
        if (geoAnimatableAnimationState.isMoving()) {
            geoAnimatableAnimationState.getController().setAnimation(RawAnimation.begin().then("flying", Animation.LoopType.LOOP));
            return PlayState.CONTINUE;
        }

        geoAnimatableAnimationState.getController().setAnimation(RawAnimation.begin().then("idle", Animation.LoopType.LOOP));
        return PlayState.CONTINUE;
    }

    @Override
    public AnimatableInstanceCache getAnimatableInstanceCache() {
        return cache;
    }

    public class RandomFlyGoal extends class_1352 {
        private final MonarchButterfly butterfly;
        private final double speed;
        private final Random random = new Random();
        private class_243 target;

        public RandomFlyGoal(MonarchButterfly butterfly, double speed) {
            this.butterfly = butterfly;
            this.speed = speed;
            this.method_6265(EnumSet.of(class_4134.field_18405));
        }

        @Override
        public boolean method_6264() {
            return !butterfly.method_5962().method_6241() && random.nextInt(10) == 0;
        }

        @Override
        public boolean method_6266() {
            return target != null && butterfly.method_19538().method_1022(target) > 0.5;
        }

        @Override
        public void method_6269() {
            target = getRandomLocation();
            butterfly.method_5962().method_6239(target.field_1352, target.field_1351, target.field_1350, speed);
        }

        private class_243 getRandomLocation() {
            // Try up to 10 times to find a valid location
            for (int attempts = 0; attempts < 10; attempts++) {
                double x = butterfly.method_23317() + (random.nextDouble() * 10 - 5);
                double z = butterfly.method_23321() + (random.nextDouble() * 10 - 5);

                double currentY = butterfly.method_23318();
                double worldBottom = butterfly.method_37908().method_31607();
                double worldTop = butterfly.method_37908().method_31600();

                // Define safe vertical bounds
                double safeMinY = worldBottom + 20;  // e.g., at least 20 blocks above the bottom
                double safeMaxY = worldTop - 5;        // slightly below the top

                // Try to keep the new Y close to the current Y (±3), clamped to our safe boundaries
                double minY = Math.max(currentY - 3, safeMinY);
                double maxY = Math.min(currentY + 3, safeMaxY);
                if (minY > maxY) { // in case the range inverted
                    minY = safeMinY;
                    maxY = safeMinY + 6;
                }
                double y = minY + random.nextDouble() * (maxY - minY);

                class_243 candidate = new class_243(x, y, z);
                // Check if the candidate location is in a non-solid block (air)
                if (butterfly.method_37908().method_8320(new class_2338((int) candidate.field_1352, (int) candidate.field_1351, (int) candidate.field_1350)).method_26215()) {
                    return candidate;
                }
            }
            // If no valid position is found after several attempts, return current position as fallback.
            return butterfly.method_19538();
        }



    }


    class ButterflyMoveControl extends class_1335 {
        private final MonarchButterfly butterfly;

        public ButterflyMoveControl(MonarchButterfly butterfly) {
            super(butterfly);
            this.butterfly = butterfly;
        }

        @Override
        public void method_6240() {
            if (this.field_6374 == class_1336.field_6378) {
                class_243 target = new class_243(this.field_6370, this.field_6369, this.field_6367);
                class_243 direction = target.method_1020(butterfly.method_19538()).method_1029();
                double speed = 0.05; // Slightly increase speed for more natural movement

                // Apply movement smoothly, allowing vertical movement
                butterfly.method_18799(new class_243(direction.field_1352 * speed, direction.field_1351 * speed, direction.field_1350 * speed));

                // Only recalc rotation if the direction isn't near zero
                if (direction.method_1027() > 1e-6) {
                    // Calculate yaw: arctan2 returns radians; convert to degrees and adjust so 0 faces north
                    float newYaw = (float)(Math.atan2(direction.field_1350, direction.field_1352) * (180 / Math.PI)) - 90F;
                    // Calculate pitch: negative arctan2 of vertical vs horizontal component
                    float newPitch = (float)(-Math.atan2(direction.field_1351, Math.sqrt(direction.field_1352 * direction.field_1352 + direction.field_1350 * direction.field_1350)) * (180 / Math.PI));

                    butterfly.method_36456(newYaw);
                    butterfly.method_36457(newPitch);
                }

                // Stop moving when close to the target
                if (butterfly.method_19538().method_1022(target) < 0.5) {
                    this.field_6374 = class_1336.field_6377;
                }
            }
        }
    }
}
